use super::*;
use super::game::*;
use super::renderer::*;

const TILE_PLAYER_DOWN : (u8, u8) = (0, 1);
const TILE_PLAYER_UP : (u8, u8) = (1, 1);
const TILE_PLAYER_LEFT : (u8, u8) = (2, 1);
const TILE_PLAYER_RIGHT : (u8, u8) = (3, 1);

const TILE_LIT_PLAYER_DOWN : (u8, u8) = (0, 2);
const TILE_LIT_PLAYER_UP : (u8, u8) = (1, 2);
const TILE_LIT_PLAYER_LEFT : (u8, u8) = (2, 2);
const TILE_LIT_PLAYER_RIGHT : (u8, u8) = (3, 2);

const TILE_MONSTER_DOWN : (u8, u8) = (5, 1);
const TILE_MONSTER_UP : (u8, u8) = (6, 1);
const TILE_MONSTER_RIGHT : (u8, u8) = (7, 1);
const TILE_MONSTER_LEFT : (u8, u8) = (8, 1);


type Frame = ((u8, u8), i8, i8);
type Frames = &'static [Frame];

pub const ANIM_PLAYER_STANDS_STILL : Frames = &[
    (TILE_PLAYER_DOWN, 0, 0),
];

pub const ANIM_PLAYER_WALKS_LEFT : Frames = &[
    (TILE_PLAYER_LEFT, 14, 0),
    (TILE_PLAYER_LEFT, 12, 1),
    (TILE_PLAYER_LEFT, 10, 1),
    (TILE_PLAYER_LEFT, 8, 2),
    (TILE_PLAYER_LEFT, 6, 2),
    (TILE_PLAYER_LEFT, 4, 1),
    (TILE_PLAYER_LEFT, 2, 1),
    (TILE_PLAYER_LEFT, 0, 0),
];

pub const ANIM_PLAYER_WALKS_RIGHT : Frames = &[
    (TILE_PLAYER_RIGHT, -14, 0),
    (TILE_PLAYER_RIGHT, -12, 1),
    (TILE_PLAYER_RIGHT, -10, 1),
    (TILE_PLAYER_RIGHT, -8, 2),
    (TILE_PLAYER_RIGHT, -6, 2),
    (TILE_PLAYER_RIGHT, -4, 1),
    (TILE_PLAYER_RIGHT, -2, 1),
    (TILE_PLAYER_RIGHT, 0, 0),
];

pub const ANIM_PLAYER_WALKS_DOWN : Frames = &[
    (TILE_PLAYER_DOWN, 0, 14),
    (TILE_PLAYER_DOWN, 0, 12+1),
    (TILE_PLAYER_DOWN, 0, 10+1),
    (TILE_PLAYER_DOWN, 0, 8+2),
    (TILE_PLAYER_DOWN, 0, 6+2),
    (TILE_PLAYER_DOWN, 0, 4+1),
    (TILE_PLAYER_DOWN, 0, 2+1),
    (TILE_PLAYER_DOWN, 0, 0),
];

pub const ANIM_PLAYER_WALKS_UP : Frames = &[
    (TILE_PLAYER_UP, 0, -14),
    (TILE_PLAYER_UP, 0, -12+1),
    (TILE_PLAYER_UP, 0, -10+1),
    (TILE_PLAYER_UP, 0, -8+2),
    (TILE_PLAYER_UP, 0, -6+2),
    (TILE_PLAYER_UP, 0, -4+1),
    (TILE_PLAYER_UP, 0, -2+1),
    (TILE_PLAYER_UP, 0, 0),
];

pub const ANIM_LIT_PLAYER_WALKS_LEFT : Frames = &[
    (TILE_LIT_PLAYER_LEFT, 14, 0),
    (TILE_LIT_PLAYER_LEFT, 12, 1),
    (TILE_LIT_PLAYER_LEFT, 10, 1),
    (TILE_LIT_PLAYER_LEFT, 8, 2),
    (TILE_LIT_PLAYER_LEFT, 6, 2),
    (TILE_LIT_PLAYER_LEFT, 4, 1),
    (TILE_LIT_PLAYER_LEFT, 2, 1),
    (TILE_LIT_PLAYER_LEFT, 0, 0),
];

pub const ANIM_LIT_PLAYER_WALKS_RIGHT : Frames = &[
    (TILE_LIT_PLAYER_RIGHT, -14, 0),
    (TILE_LIT_PLAYER_RIGHT, -12, 1),
    (TILE_LIT_PLAYER_RIGHT, -10, 1),
    (TILE_LIT_PLAYER_RIGHT, -8, 2),
    (TILE_LIT_PLAYER_RIGHT, -6, 2),
    (TILE_LIT_PLAYER_RIGHT, -4, 1),
    (TILE_LIT_PLAYER_RIGHT, -2, 1),
    (TILE_LIT_PLAYER_RIGHT, 0, 0),
];

pub const ANIM_LIT_PLAYER_WALKS_DOWN : Frames = &[
    (TILE_LIT_PLAYER_DOWN, 0, 14),
    (TILE_LIT_PLAYER_DOWN, 0, 12+1),
    (TILE_LIT_PLAYER_DOWN, 0, 10+1),
    (TILE_LIT_PLAYER_DOWN, 0, 8+2),
    (TILE_LIT_PLAYER_DOWN, 0, 6+2),
    (TILE_LIT_PLAYER_DOWN, 0, 4+1),
    (TILE_LIT_PLAYER_DOWN, 0, 2+1),
    (TILE_LIT_PLAYER_DOWN, 0, 0),
];

pub const ANIM_LIT_PLAYER_WALKS_UP : Frames = &[
    (TILE_LIT_PLAYER_UP, 0, -14),
    (TILE_LIT_PLAYER_UP, 0, -12+1),
    (TILE_LIT_PLAYER_UP, 0, -10+1),
    (TILE_LIT_PLAYER_UP, 0, -8+2),
    (TILE_LIT_PLAYER_UP, 0, -6+2),
    (TILE_LIT_PLAYER_UP, 0, -4+1),
    (TILE_LIT_PLAYER_UP, 0, -2+1),
    (TILE_LIT_PLAYER_UP, 0, 0),
];


pub const ANIM_MONSTER_WALKS_LEFT : Frames = &[
    (TILE_MONSTER_LEFT, 14, 0),
    (TILE_MONSTER_LEFT, 12, 1),
    (TILE_MONSTER_LEFT, 10, 1),
    (TILE_MONSTER_LEFT, 8, 2),
    (TILE_MONSTER_LEFT, 6, 2),
    (TILE_MONSTER_LEFT, 4, 1),
    (TILE_MONSTER_LEFT, 2, 1),
    (TILE_MONSTER_LEFT, 0, 0),
];

pub const ANIM_MONSTER_WALKS_RIGHT : Frames = &[
    (TILE_MONSTER_RIGHT, -14, 0),
    (TILE_MONSTER_RIGHT, -12, 1),
    (TILE_MONSTER_RIGHT, -10, 1),
    (TILE_MONSTER_RIGHT, -8, 2),
    (TILE_MONSTER_RIGHT, -6, 2),
    (TILE_MONSTER_RIGHT, -4, 1),
    (TILE_MONSTER_RIGHT, -2, 1),
    (TILE_MONSTER_RIGHT, 0, 0),
];

pub const ANIM_MONSTER_WALKS_DOWN : Frames = &[
    (TILE_MONSTER_DOWN, 0, 14),
    (TILE_MONSTER_DOWN, 0, 12+1),
    (TILE_MONSTER_DOWN, 0, 10+1),
    (TILE_MONSTER_DOWN, 0, 8+2),
    (TILE_MONSTER_DOWN, 0, 6+2),
    (TILE_MONSTER_DOWN, 0, 4+1),
    (TILE_MONSTER_DOWN, 0, 2+1),
    (TILE_MONSTER_DOWN, 0, 0),
];

pub const ANIM_MONSTER_WALKS_UP : Frames = &[
    (TILE_MONSTER_UP, 0, -14),
    (TILE_MONSTER_UP, 0, -12+1),
    (TILE_MONSTER_UP, 0, -10+1),
    (TILE_MONSTER_UP, 0, -8+2),
    (TILE_MONSTER_UP, 0, -6+2),
    (TILE_MONSTER_UP, 0, -4+1),
    (TILE_MONSTER_UP, 0, -2+1),
    (TILE_MONSTER_UP, 0, 0),
];

pub const ANIM_MONSTER_ATTACKS : Frames = &[
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
    (TILE_MONSTER_UP, 0, 0),
];

pub const ANIM_FLAME : Frames = &[
    ((4, 0), 0, 2),
    ((5, 0), 0, 2),
    ((6, 0), 0, 2),
    ((7, 0), 0, 2),
    ((8, 0), 0, 2),
    ((9, 0), 0, 2),
    ((10, 0), 0, 2),
    ((11, 0), 0, 2),
];

pub const ANIM_MONSTER_RISES : Frames = &[
    ((9, 1), 0, 0),
    ((10, 1), 0, 0),
    ((11, 1), 0, 0),
    ((12, 1), 0, 0),
    ((13, 1), 0, 0),
    ((14, 1), 0, 0),
    ((15, 1), 0, 0),
    ((16, 1), 0, 0),
];

pub const ANIM_MONSTER_DESTROYED : Frames = &[
    ((17, 1), 0, 0),
    ((18, 1), 0, 0),
    ((19, 1), 0, 0),
    ((20, 1), 0, 0),
    ((21, 1), 0, 0),
    ((22, 1), 0, 0),
    ((23, 1), 0, 0),
    ((24, 1), 0, 0),
];

pub const ANIM_PLAYER_AFLAME : Frames = &[
    ((0, 3), 0, 0),
    ((1, 3), 0, 0),
    ((2, 3), 0, 0),
    ((3, 3), 0, 0),
    ((4, 3), 0, 0),
    ((5, 3), 0, 0),
    ((6, 3), 0, 0),
    ((7, 3), 0, 0),
    ((0, 2), 0, 0),
];

pub const ANIM_UI_PUMPKIN : Frames = &[
    ((0, 3), 0, 2),
    ((1, 3), 0, 2),
    ((2, 3), 0, 2),
    ((3, 3), 0, 2),
    ((4, 3), 0, 2),
    ((5, 3), 0, 2),
    ((6, 3), 0, 2),
    ((7, 3), 0, 2),
    ((0, 2), 0, 2),
];


pub const ANIM_PLAYER_STRUGGLE : Frames = &[
    ((4, 2), 0, 0),
    ((5, 2), 0, 0),
    ((4, 2), 0, 0),
    ((5, 2), 0, 0),
    ((4, 2), 0, 0),
    ((5, 2), 0, 0),
    ((4, 2), 0, 0),
    ((5, 2), 0, 0),
];

pub const ANIM_VICTORY : Frames = &[
    ((0, 3), 0, 0),
    ((1, 3), 0, 2),
    ((2, 3), 0, 4),
    ((3, 3), 0, 5),
    ((4, 3), 0, 6),
    ((5, 3), 0, 5),
    ((6, 3), 0, 4),
    ((7, 3), 0, 2),
    ((0, 2), 0, 0),
];


pub struct Animation {
    frames : Frames,
    current : u8,
    looping: bool,
}

impl Animation {
    pub fn new(frames: Frames, looping: bool) -> Self {
        Animation {
            frames,
            current: 0,
            looping,
        }
    }
    pub fn current_frame(&self) -> Option<&Frame> {
        self.frames.get(self.current as usize)
    }
    pub fn done(&self) -> bool {
        return self.current_frame() == None;
    }

    pub fn tick(&mut self) {
        if !self.done() {
            self.current += 1;
        }
        if self.looping && self.done() {
            self.current = 0;
        }
    }
    pub fn restart(&mut self) {
        self.current = 0;
    }
    pub fn render(&self, buffer : &mut Vec<Vertex>, tile: (u16, u16), light : (u8, u8, u8, u8)) {
        if let Some(&(sprite, dx, dy)) = self.current_frame().or(self.frames.last()) {
            let (x, y) = tile;
            let x = x * 16;
            let y = y * 16;
            if dx < 0 && (-dx) as u16 > x {return};
            if dy < 0 && (-dy) as u16 > y {return};
            let dx = dx as i16 as u16;
            let dy = dy as i16 as u16;
            define_rect(
                buffer, 
                (x + dx, y + dy),
                sprite,
                light);
        }
    }
}

